﻿Option Strict On
Option Infer On

''' <summary>
''' Serial Port Logger
''' </summary>
''' <remarks></remarks>
Public Class mainForm

    Private readBuffer As String = String.Empty
    Private comOpen As Boolean
    Private file As System.IO.StreamWriter

#Region "form events"
    ''' <summary>
    ''' close application and COM Port
    ''' </summary>
    Private Sub Form1_FormClosed(ByVal sender As System.Object, _
                                 ByVal e As System.Windows.Forms.FormClosedEventArgs) _
                                 Handles MyBase.FormClosed
        If comOpen Then
            SerialPort1.Close()
            file.Close()
        End If
    End Sub

    ''' <summary>
    ''' open Windows Form
    ''' </summary>
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Dim Portnames As String() = System.IO.Ports.SerialPort.GetPortNames
        If Portnames.Count = 0 Then
            MsgBox("There are no Com Ports detected!")
            Me.Close()
        End If
        cboComPort.Items.AddRange(Portnames)
        cboComPort.Text = Portnames(0)
        cboBaudRate.Text = "115200"

    End Sub

    ''' <summary>
    ''' Open Com Port here
    ''' </summary>
    Private Sub btnComOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnComOpen.Click
        If SaveFileDialog1.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
            If SaveFileDialog1.FileName <> "" Then
                lblFileName.Text = "Data saved to: " + SaveFileDialog1.FileName
                file = My.Computer.FileSystem.OpenTextFileWriter(SaveFileDialog1.FileName, True)
                ' device params
                With SerialPort1
                    .ParityReplace = &H3B                    ' replace ";" when parity error occurs
                    .PortName = cboComPort.Text
                    .BaudRate = CInt(cboBaudRate.Text)
                    .Parity = IO.Ports.Parity.None
                    .DataBits = 8
                    .StopBits = IO.Ports.StopBits.One
                    .Handshake = IO.Ports.Handshake.None
                    .RtsEnable = False
                    .ReceivedBytesThreshold = 1             'threshold: one byte in buffer > event is fired
                    .NewLine = vbCr         ' CR must be the last char in frame. This terminates the SerialPort.readLine
                    .ReadTimeout = 1000
                End With
                ' check whether device is avaiable:
                Try
                    SerialPort1.Open()
                    comOpen = SerialPort1.IsOpen
                Catch ex As Exception
                    comOpen = False
                    MsgBox("Error Open: " & ex.Message)
                End Try

                If comOpen Then
                    btnComOpen.Enabled = False
                    btnComClose.Enabled = True
                    cboComPort.Enabled = False
                    cboBaudRate.Enabled = False
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' close ComPort
    ''' </summary>
    Private Sub btnComClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnComClose.Click
        'Dim temp As Long
        If comOpen Then
            ' clear input buffer
            comOpen = False
            SerialPort1.DiscardInBuffer()
            Application.DoEvents()
            SerialPort1.Close()
            file.Close()
            btnComClose.Enabled = False
            btnComOpen.Enabled = True
        End If
        comOpen = False
        cboComPort.Enabled = True
        cboBaudRate.Enabled = True
    End Sub

#End Region

#Region "ComPort read data"

    ''' <summary>
    ''' async read on secondary thread
    ''' </summary>
    Private Sub SerialPort1_DataReceived(ByVal sender As System.Object, _
                                         ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) _
                                         Handles SerialPort1.DataReceived
        If comOpen Then
            Try
                ' Read entire string until .Newline 
                readBuffer = SerialPort1.ReadLine()
                ' Append time stamp ?
                If ckbTimeStamp.Checked Then
                    ' Append timestamp to data received
                    readBuffer = readBuffer + "," + DateTimeOffset.Now.ToString("G")
                End If
                ' Write data to log file
                file.WriteLine(readBuffer)
                'Data to UI thread
                Me.Invoke(New EventHandler(AddressOf DoUpdate))
            Catch ex As Exception
                MsgBox("read " & ex.Message)
            End Try
        End If
    End Sub


    ''' <summary>
    ''' update received string in UI
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub DoUpdate(ByVal sender As Object, ByVal e As System.EventArgs)
        tbRx.Text = readBuffer
    End Sub


#End Region

End Class
